import asyncio

from pydeconz import DeconzSession
from pydeconz.utils import (get_api_key, delete_all_keys)
from .websocket import WSClient

@asyncio.coroutine
def main(loop, **kwargs):
    """
    """
    if 'api_key' not in kwargs:
        api_key = yield from get_api_key(loop, **kwargs)
        kwargs['api_key'] = api_key
        print(api_key)

    deconz = DeconzSession(loop, **kwargs)
    result = yield from deconz.populate_config()
    result = yield from deconz.populate_lights()
    result = yield from deconz.populate_groups()
    result = yield from deconz.populate_sensors()
    deconz.start()
    from pprint import pprint
    for dev_id, dev in deconz.groups.items():
        pprint(dev.__dict__)
    #for light_id, light in deconz.lights.items():
    #    yield from light.set_state({'hue': 5500})
#    print(deconz.lights)
#    field = '/lights/1/state'
#    data = {'on': False}
#    yield from deconz.lights['1'].set_state(field, data)
    #while True:
    #    yield from deconz.get_event_async()
    #    pass
    #try:
    #    yield from delete_all_keys(loop, **kwargs)
    #finally:
    #    pass
#    yield from deconz.close()
    #yield from delete_api_key(loop, **kwargs)


kw = {'host': '10.0.1.16',
      'port': 80,
      'api_key': 'F0FB5939EF',
      'username': 'delight',
      'password': 'delight'}

loop = asyncio.get_event_loop()
loop.run_until_complete(main(loop, **kw))
# loop.call_soon(partial(main, loop, **kw))
loop.run_forever()
loop.close()



"""Python library to connect Deconz and Home Assistant to work together."""

import asyncio
import logging
import aiohttp
import async_timeout

_LOGGER = logging.getLogger(__name__)


@asyncio.coroutine
def get_api_key(loop, host, port, username, password, **kwargs):
    """Get a new API key for devicetype."""
    url = 'http://' + host + ':' + str(port) + '/api'
    auth = aiohttp.BasicAuth(username, password=password)
    data = b'{"devicetype": "pydeconz"}'
    session = aiohttp.ClientSession(loop=loop)
    response = yield from request(session.post, url, auth=auth, data=data)
    yield from session.close()
    if response:
        api_key = response[0]['success']['username']
        _LOGGER.info('API key: %s', api_key)
        return api_key
    else:
        return False


@asyncio.coroutine
def delete_api_key(loop, host, port, api_key, **kwargs):
    """Delete API key from Deconz."""
    url = 'http://' + host + ':' + str(port) + '/api' + api_key + '/config/whitelist/' + api_key
    session = aiohttp.ClientSession(loop=loop)
    response = yield from request(session.delete, url)
    yield from session.close()
    if response:
        _LOGGER.info(response)


@asyncio.coroutine
def delete_all_keys(loop, host, port, api_key, **kwargs):
    """Delete all API keys except for the one provided to the method."""
    url = 'http://' + host + ':' + str(port) + '/api' + api_key + '/config'
    session = aiohttp.ClientSession(loop=loop)
    response = yield from request(session.get, url)
    yield from session.close()
    for key, _ in response['whitelist'].items():
        if key != api_key:
            yield from delete_api_key(loop, host, port, key)


@asyncio.coroutine
def request(session, url, **kwargs):
    """Do a web request and manage response."""
    try:
        with async_timeout.timeout(10):
            _LOGGER.debug('Sending %s to %s', kwargs, url)
            response = yield from session(url, **kwargs)
        if response.status != 200:
            _LOGGER.error("HTTP status %d, response %s.",
                          response.status, (yield from response.text()))
            return False
        result = yield from response.json()
    except asyncio.TimeoutError:
        _LOGGER.error("Timeout getting DeConz data from %s.", url)
        return False
    except aiohttp.ClientError:
        _LOGGER.error("Error getting DeConz data from %s.", url)
        return False
    else:
        _LOGGER.debug('HTTP request response: %s', result)
        return result



"""Python library to connect Deconz and Home Assistant to work together."""

import asyncio
import json
import logging
import aiohttp

from .config import DeconzConfig
from .group import DeconzGroup, DeconzScene
from .light import DeconzLight
from .sensor import create_sensor
from .utils import request
from .websocket import WSClient

_LOGGER = logging.getLogger(__name__)

class DeconzSession:
    """Deconz representation that handles lights and sensors."""

    def __init__(self, loop, host, port, api_key, **kwargs):
        """Setup session and host information."""
        self.groups = {}
        self.lights = {}
        self.scenes = {}
        self.sensors = {}
        self.config = None
        self.loop = loop
        self.session = aiohttp.ClientSession(loop=loop)
        self.api_url = 'http://%s:%d/api/%s' % (host, port, api_key)
        self.websocket = None

    def start(self):
        """Connect websocket to Deconz."""
        if self.config:
            self.websocket = WSClient(self.loop,
                                      self.config.host,
                                      self.config.websocketport,
                                      self.event_handler)
            self.websocket.start()
        else:
            _LOGGER.error('No Deconz config available')

    def close(self):
        """Close websession and websocket to Deconz."""
        _LOGGER.info('Shutting down connections to Deconz.')
        self.session.close()
        if self.websocket:
            self.websocket.stop()

    @asyncio.coroutine
    def populate_config(self):
        """Load Deconz configuration parameters."""
        config = yield from self.get_state_async('/config')
        if config:
            self.config = DeconzConfig(config)

    @asyncio.coroutine
    def populate_groups(self):
        """Create lights based on all lights registered in Deconz."""
        groups = yield from self.get_state_async('/groups')
        if groups:
            for group_id, group in groups.items():
                self.groups[group_id] = DeconzGroup(group_id, group, self.put_state_async)

    @asyncio.coroutine
    def populate_lights(self):
        """Create lights based on all lights registered in Deconz."""
        lights = yield from self.get_state_async('/lights')
        if lights:
            for light_id, light in lights.items():
                self.lights[light_id] = DeconzLight(light_id, light, self.put_state_async)

    #@asyncio.coroutine
    def populate_scenes(self):
        """Create lights based on all lights registered in Deconz."""
        #groups = yield from self.get_state_async('/groups')
        if self.groups:
            for group_id, group in self.groups.items():
                for scene in group.scenes:
                    scene = DeconzScene(group, scene, self.put_state_async)
                    self.scenes[group_id + '_' + scene.id] = scene
            print(self.scenes)

    @asyncio.coroutine
    def populate_sensors(self):
        """Create sensors based on all sensors registered in Deconz."""
        sensors = yield from self.get_state_async('/sensors')
        if sensors:
            for sensor_id, sensor in sensors.items():
                self.sensors[sensor_id] = create_sensor(sensor)

    @asyncio.coroutine
    def put_state_async(self, field, data):
        """Set state of object in Deconz.

        Field is a string representing a specific device in Deconz
        e.g. field='/lights/1/state'.
        Data is a json object with what data you want to alter
        e.g. data={'on': True}.
        See Dresden Elektroniks REST API documentation for details:
        http://dresden-elektronik.github.io/deconz-rest-doc/rest/
        """
        session = self.session.put
        url = self.api_url + field
        jsondata = json.dumps(data)
        response_dict = yield from request(session, url, data=jsondata)
        return response_dict

    @asyncio.coroutine
    def get_state_async(self, field):
        """Get state of object in Deconz.

        Field is a string representing an API endpoint or lower
        e.g. field='/lights'.
        See Dresden Elektroniks REST API documentation for details:
        http://dresden-elektronik.github.io/deconz-rest-doc/rest/
        """
        session = self.session.get
        url = self.api_url + field
        response_dict = yield from request(session, url)
        return response_dict

    def event_handler(self, event):
        """Receive event from websocket and identifies where the event belong.

        {
            "t": "event",
            "e": "changed",
            "r": "sensors",
            "id": "12",
            "state": { "buttonevent": 2002 }
        }
        """
        if event['e'] == 'changed':
            if event['r'] == 'groups' and event['id'] in self.groups:
                self.groups[event['id']].update(event)
            elif event['r'] == 'lights' and event['id'] in self.lights:
                self.lights[event['id']].update(event)
            elif event['r'] == 'sensors' and event['id'] in self.sensors:
                self.sensors[event['id']].update(event)
            else:
                _LOGGER.debug('Not supported event %s', event)
        else:
            _LOGGER.debug('Not supported event %s', event)
            
